@@ -1,5 +1,7 @@
-enum event_enum { KEYBOARD_INTERRUPT_EVENT, IO_COMPLETION_EVENT, ALARM_EVENT,
-		    OS_SIGNAL_EVENT, ERROR_EVENT, NO_EVENT };
+enum event_enum { KEYBOARD_INTERRUPT_EVENT,
+		  IO_READ_COMPLETION_EVENT, IO_WRITE_COMPLETION_EVENT,
+		  ALARM_EVENT,
+		  OS_SIGNAL_EVENT, ERROR_EVENT, NO_EVENT };
 
 extern bool s48_add_pending_fd(int fd, bool is_input);
 extern bool s48_remove_fd(int fd);
