@@ -32,6 +32,7 @@
 
 
 char ** process_args(char **argv,
+                     char* prog_name,
 		     long *heap_size,
 		     long *stack_size,
 		     char **object_file,
@@ -65,7 +66,7 @@ main(argc, argv)
   char *me = *argv;		/* Save program name. */
   prog_name = *argv++;
 
-  argv=process_args(argv, 		
+  argv=process_args(argv, prog_name, 		
 		    &heap_size, &stack_size,
 		    &object_file, &image_name);
   for(argc=0, argp=argv; *argp; argc++, argp++); /* Recompute argc. */
