@@ -345,10 +345,8 @@
 
 ;;; All you have to do, if a wait on proc was successful
 (define (waited-by-wait proc status)
-  (if (not (status:stop-sig status))
-      (begin
-	(obituary proc status)
-	(mark-proc-waited! proc))))
+  (obituary proc status)
+  (mark-proc-waited! proc))
 
 ;;; we know from somewhere that proc is dead
 (define (obituary proc status)
