@@ -22,8 +22,6 @@
 ;;; is the char-set parsing and unparsing, which deal with ranges of 
 ;;; characters. We assume an 8-bit ASCII superset.
 
-;;; Imports:
-;;; ? for COND, and SWITCHQ conditional form.
 ;;; every
 
 ;;; This code is much hairier than it would otherwise be because of the
@@ -52,23 +50,24 @@
 ;;; in the form of embedded code in some of the regexp's fields?
 
 (define (static-regexp? re)
-  (? ((re-seq?    re)   (every static-regexp? (re-seq:elts    re)))
-     ((re-choice? re)   (every static-regexp? (re-choice:elts re)))
+  (cond
+   ((re-seq?    re)   (every static-regexp? (re-seq:elts    re)))
+   ((re-choice? re)   (every static-regexp? (re-choice:elts re)))
+   
+   ((re-char-set? re) (char-set? (re-char-set:cset re))) ; Might be code.
 
-     ((re-char-set? re) (char-set? (re-char-set:cset re)))    ; Might be code.
+   ((re-repeat? re)			; FROM & TO fields might be code.
+    (let ((to (re-repeat:to re)))
+      (and (integer? (re-repeat:from re))
+	   (or (not to) (integer? to))
+	   (static-regexp? (re-repeat:body re)))))
 
-     ((re-repeat? re)		; FROM & TO fields might be code.
-      (let ((to (re-repeat:to re)))
-	(and (integer? (re-repeat:from re))
-	     (or (not to) (integer? to))
-	     (static-regexp? (re-repeat:body re)))))
+   ((re-dsm? re)      (static-regexp? (re-dsm:body re)))
+   ((re-submatch? re) (static-regexp? (re-submatch:body re)))
 
-     ((re-dsm? re)      (static-regexp? (re-dsm:body re)))
-     ((re-submatch? re) (static-regexp? (re-submatch:body re)))
-
-     (else (or (re-bos? re) (re-eos? re) ; Otw, if it's not 
-	       (re-bol? re) (re-eol? re) ; one of these, ; then it's Scheme code.
-	       (re-string? re))))) 
+   (else (or (re-bos? re) (re-eos? re)	; Otw, if it's not 
+	     (re-bol? re) (re-eol? re)	; one of these, ; then it's Scheme code.
+	     (re-string? re))))) 
                
 
 ;;; Two useful standard char sets
@@ -143,141 +142,173 @@
       (define (non-cset)	; Blow up if cset? is true.
 	(if cset? (error "Illegal SRE in char-class context." sre)))
 
-      (? ((char? sre)   (parse-char-re   sre case-sensitive? cset?))
-	 ((string? sre) (parse-string-re sre case-sensitive? cset?))
+      (define (build-re-repeat from to stuff)
+	(let ((seq (parse-seq stuff)))
+	  (if (static-regexp? seq)
+	      (re-repeat from to seq)
+	      `(,(r 're-repeat) ',from ',to ,(regexp->scheme seq r)))))
 
-	 ((c sre %bos) (non-cset) re-bos)
-	 ((c sre %eos) (non-cset) re-eos)
+      (cond
+       ((char? sre)   (parse-char-re   sre case-sensitive? cset?))
+       ((string? sre) (parse-string-re sre case-sensitive? cset?))
 
-	 ((c sre %bol) (non-cset) re-bol)
-	 ((c sre %eol) (non-cset) re-eol)
+       ((c sre %bos) (non-cset) re-bos)
+       ((c sre %eos) (non-cset) re-eos)
 
-	 ((pair? sre)
-	  (let ((hygn-eq? (lambda (the-sym) (c (car sre) (r the-sym)))))
-	    (cond 
-	     ((hygn-eq? '*) (non-cset) (re-repeat 0 #f (parse-seq (cdr sre))))
-	     ((hygn-eq? '+)  (non-cset) (re-repeat 1 #f (parse-seq (cdr sre))))
-	     ((hygn-eq? '?)  (non-cset) (re-repeat 0  1 (parse-seq (cdr sre))))
-	     ((hygn-eq? '=)  (non-cset) (let ((n (cadr sre)))
-					  (re-repeat n n (parse-seq (cddr sre)))))
-	     ((hygn-eq? '>=) (non-cset) (re-repeat (cadr sre) #f (parse-seq (cddr sre))))
-	     ((hygn-eq? '**) (non-cset) (re-repeat (cadr sre) (caddr sre)
-						   (parse-seq (cdddr sre))))
+       ((c sre %bol) (non-cset) re-bol)
+       ((c sre %eol) (non-cset) re-eol)
+
+       ((pair? sre)
+	(let ((hygn-eq? (lambda (the-sym) (or (c (car sre) (r the-sym))
+					      (c (car sre) the-sym)))))
+	  (cond 
+	   ((hygn-eq? '*)
+	    (non-cset)
+	    (build-re-repeat 0 #f (cdr sre)))
+	   ((hygn-eq? '+)
+	    (non-cset)
+	    (build-re-repeat 1 #f (cdr sre)))
+	   ((hygn-eq? '?) 
+	    (non-cset)
+	    (build-re-repeat 0 1 (cdr sre)))
+	   ((hygn-eq? '=)
+	    (non-cset)
+	    (let ((n (cadr sre)))
+	      (build-re-repeat n n (cddr sre))))
+	   ((hygn-eq? '>=)
+	    (non-cset)
+	    (build-re-repeat (cadr sre) #f (cddr sre)))
+	   ((hygn-eq? '**)
+	    (non-cset)
+	    (build-re-repeat (cadr sre) (caddr sre)
+			     (cdddr sre)))
 	     
-	     ;; Choice is special wrt cset? because it's "polymorphic".
-	     ;; Note that RE-CHOICE guarantees to construct a char-set
-	     ;; or single-char string regexp if all of its args are char 
-	     ;; classes.
-	     ((or (hygn-eq? '|)
-		  (hygn-eq? 'or))
-	      (let ((elts (map (lambda (sre)
-				 (recur sre case-sensitive? cset?))
-			       (cdr sre))))
-		(if cset?
-		    (assoc-cset-op char-set-union 'char-set-union elts r)
-		    (re-choice elts))))
+	   ;; Choice is special wrt cset? because it's "polymorphic".
+	   ;; Note that RE-CHOICE guarantees to construct a char-set
+	   ;; or single-char string regexp if all of its args are char 
+	   ;; classes.
+	   ((or (hygn-eq? '|)
+		(hygn-eq? 'or))
+	    (let ((elts (map (lambda (sre)
+			       (recur sre case-sensitive? cset?))
+			     (cdr sre))))
+	      (if cset?
+		  (assoc-cset-op char-set-union 'char-set-union elts r)
+		  (re-choice elts))))
 	     
-	     ((or (hygn-eq? ':)
-		  (hygn-eq? 'seq))
-	      (non-cset) (parse-seq (cdr sre)))
+	   ((or (hygn-eq? ':)
+		(hygn-eq? 'seq))
+	    (non-cset)
+	    (parse-seq (cdr sre)))
 	     
-	     ((hygn-eq? 'submatch) (non-cset) (re-submatch (parse-seq (cdr sre))))
-	     ((hygn-eq? 'dsm)      (non-cset) (re-dsm (parse-seq (cdddr sre))
-						      (cadr sre)
-						      (caddr sre)))
+	   ((hygn-eq? 'submatch)
+	    (non-cset)
+	    (let ((seq (parse-seq (cdr sre))))
+	      (if (static-regexp? seq)
+		  (re-submatch seq)
+		  `(,(r 're-submatch) ,(regexp->scheme seq r)))))
+	   ((hygn-eq? 'dsm)
+	    (non-cset)
+	    (re-dsm (parse-seq (cdddr sre))
+		    (cadr sre)
+		    (caddr sre)))
 	     
-	     ;; We could be more aggressive and push the uncase op down into
-	     ;; partially-static regexps, but enough is enough.
-	     ((hygn-eq? 'uncase)
-	      (let ((re-or-cset (parse-seq (cdr sre))))	; Depending on CSET?.
-		(if cset?
+	   ;; We could be more aggressive and push the uncase op down into
+	   ;; partially-static regexps, but enough is enough.
+	   ((hygn-eq? 'uncase)
+	    (let ((re-or-cset (parse-seq (cdr sre)))) ; Depending on CSET?.
+	      (if cset?
 		    
-		    (if (re-char-set? re-or-cset)	; A char set or code
-			(uncase-char-set re-or-cset)	; producing a char set.
-			`(,(r 'uncase) ,re-or-cset))
+		  (if (re-char-set? re-or-cset)	; A char set or code
+		      (uncase-char-set re-or-cset) ; producing a char set.
+		      `(,(r 'uncase) ,re-or-cset))
 		    
-		    (if (static-regexp? re-or-cset)	; A regexp or code
-			(uncase re-or-cset)		; producing a regexp.
-			`(,(r 'uncase)
-			  ,(regexp->scheme (simplify-regexp re-or-cset) r))))))
+		  (if (static-regexp? re-or-cset) ; A regexp or code
+		      (uncase re-or-cset) ; producing a regexp.
+		      `(,(r 'uncase)
+			,(regexp->scheme (simplify-regexp re-or-cset) r))))))
 	     
-	     ;; These just change the lexical case-sensitivity context.
-	     ((hygn-eq? 'w/nocase) (parse-seq/context (cdr sre) #f))
-	     ((hygn-eq? 'w/case)   (parse-seq/context (cdr sre) #t))
+	   ;; These just change the lexical case-sensitivity context.
+	   ((hygn-eq? 'w/nocase) (parse-seq/context (cdr sre) #f))
+	   ((hygn-eq? 'w/case)   (parse-seq/context (cdr sre) #t))
 	     
-	     ;; ,<exp> and ,@<exp>
-	     ((hygn-eq? 'unquote)
-	     (let ((exp (cadr sre)))
-	       (if cset?
-		   `(,%coerce-dynamic-charset ,exp)
-		   `(,%flush-submatches (,%coerce-dynamic-regexp ,exp)))))
-	     ((hygn-eq? 'unquote-splicing)
-	      (let ((exp (cadr sre)))
-		(if cset?
-		    `(,%coerce-dynamic-charset ,exp)
-		    `(,%coerce-dynamic-regexp ,exp))))
+	   ;; ,<exp> and ,@<exp>
+	   ((hygn-eq? 'unquote)
+	    (let ((exp (cadr sre)))
+	      (if cset?
+		  `(,%coerce-dynamic-charset ,exp)
+		  `(,%flush-submatches (,%coerce-dynamic-regexp ,exp)))))
+	   ((hygn-eq? 'unquote-splicing)
+	    (let ((exp (cadr sre)))
+	      (if cset?
+		  `(,%coerce-dynamic-charset ,exp)
+		  `(,%coerce-dynamic-regexp ,exp))))
 	     
-	     ((hygn-eq? '~) (let* ((cs (assoc-cset-op char-set-union 'char-set-union
-						      (map parse-char-class (cdr sre))
-						      r))
-				   (cs (if (char-set? cs)
-				(char-set-complement cs)
-				`(,(r 'char-set-complement) ,cs))))
-			      (if cset? cs (make-re-char-set cs))))
+	   ((hygn-eq? '~)
+	    (let* ((cs (assoc-cset-op char-set-union 'char-set-union
+				      (map parse-char-class (cdr sre))
+				      r))
+		   (cs (if (char-set? cs)
+			   (char-set-complement cs)
+			   `(,(r 'char-set-complement) ,cs))))
+	      (if cset? cs (make-re-char-set cs))))
 	     
-	     ((hygn-eq? '&) (let ((cs (assoc-cset-op char-set-intersection 'char-set-intersection
-						     (map parse-char-class (cdr sre))
-						     r)))
-			      (if cset? cs (make-re-char-set cs))))
+	   ((hygn-eq? '&)
+	    (let ((cs (assoc-cset-op char-set-intersection 'char-set-intersection
+				     (map parse-char-class (cdr sre))
+				     r)))
+	      (if cset? cs (make-re-char-set cs))))
 	     
-	     ((hygn-eq? '-) (if (pair? (cdr sre))
-				(let* ((cs1 (parse-char-class (cadr sre)))
-				       (cs2 (assoc-cset-op char-set-union 'char-set-union
-						(map parse-char-class (cddr sre))
-						r))
-				       (cs (if (and (char-set? cs1) (char-set? cs2))
-					       (char-set-difference cs1 cs2)
-					       `(,(r 'char-set-difference)
-						 ,(if (char-set? cs1)
-						      (char-set->scheme cs1 r)
-					   cs1)
-						 . ,(if (char-set? cs2)
-							(list (char-set->scheme cs2 r))
-							(cdr cs2))))))
-				  (if cset? cs (make-re-char-set cs)))
-				(error "SRE set-difference operator (- ...) requires at least one argument")))
+	   ((hygn-eq? '-)
+	    (if (pair? (cdr sre))
+		(let* ((cs1 (parse-char-class (cadr sre)))
+		       (cs2 (assoc-cset-op char-set-union 'char-set-union
+					   (map parse-char-class (cddr sre))
+					   r))
+		       (cs (if (and (char-set? cs1) (char-set? cs2))
+			       (char-set-difference cs1 cs2)
+			       `(,(r 'char-set-difference)
+				 ,(if (char-set? cs1)
+				      (char-set->scheme cs1 r)
+				      cs1)
+				 . ,(if (char-set? cs2)
+					(list (char-set->scheme cs2 r))
+					(list cs2))))))
+		  (if cset? cs (make-re-char-set cs)))
+		(error "SRE set-difference operator (- ...) requires at least one argument")))
 	     
-	     ((hygn-eq? '/) (let ((cset (range-class->char-set (cdr sre) case-sensitive?)))
-		   (if cset? cset (make-re-char-set cset))))
+	   ((hygn-eq? '/)
+	    (let ((cset (range-class->char-set (cdr sre) case-sensitive?)))
+	      (if cset?
+		  cset
+		  (make-re-char-set cset))))
 	     
-	     ((hygn-eq? 'posix-string)
-	     (if (and (= 1 (length (cdr sre)))
-		      (string? (cadr sre)))
-		 (posix-string->regexp (cadr sre))
-		 (error "Illegal (posix-string ...) SRE body." sre)))
+	   ((hygn-eq? 'posix-string)
+	    (if (and (= 1 (length (cdr sre)))
+		     (string? (cadr sre)))
+		(posix-string->regexp (cadr sre))
+		(error "Illegal (posix-string ...) SRE body." sre)))
 	     
-	     (else (if (every string? sre)	; A set spec -- ("wxyz").
-		       (let* ((cs (apply char-set-union
-					 (map string->char-set sre)))
-			      (cs (if case-sensitive? cs (uncase-char-set cs))))
-			 (if cset? cs (make-re-char-set cs)))
+	   (else
+	    (if (every string? sre)	; A set spec -- ("wxyz").
+		(let* ((cs (apply char-set-union
+				  (map string->char-set sre)))
+		       (cs (if case-sensitive? cs (uncase-char-set cs))))
+		  (if cset? cs (make-re-char-set cs)))
 		       
-		       (error "Illegal SRE" sre))))))
+		(error "Illegal SRE" sre))))))
 
-	 ;; It must be a char-class name (ANY, ALPHABETIC, etc.)
-	 (else
-	  (letrec ((hygn-memq? (lambda (sym-list)
-				 (if (null? sym-list) 
-				     #f
-				     (or (c sre (r (car sym-list)))
-					 (hygn-memq? (cdr sym-list)))))))
+       ;; It must be a char-class name (ANY, ALPHABETIC, etc.)
+       (else
+	(letrec ((hygn-memq? (lambda (sym-list)
+			       (if (null? sym-list) 
+				   #f
+				   (or (c sre (r (car sym-list)))
+				       (c sre (car sym-list))
+				       (hygn-memq? (cdr sym-list)))))))
 	  (let ((cs (cond
 		     ((hygn-memq? '(nonl))			nonl-chars)
 		     ((hygn-memq? '(lower-case lower))		char-set:lower-case)
-		     ;;; any is imported from scsh-utilities and for
-		     ;;; some reason this screws the hygienic renaming
-		     ;;; up here. So this is bogus:
-		     ((eq? sre 'any) char-set:full)
 		     ((hygn-memq? '(any))			char-set:full)
 		     ((hygn-memq? '(upper-case upper))		char-set:upper-case)
 		     ((hygn-memq? '(alphabetic alpha))		char-set:letter)
@@ -322,11 +353,12 @@
 (define (assoc-cset-op op op-name elts r)
   (receive (csets code-chunks) (partition char-set? elts)
     (if (pair? code-chunks)
-	(? ((pair? csets)
-	    `(,(r op-name) ,(char-set->scheme (apply op csets) r)
-			   . ,code-chunks))
-	   ((pair? (cdr code-chunks)) `(,(r op-name) . ,code-chunks))
-	   (else (car code-chunks))) ; Just one.
+	(cond
+	 ((pair? csets)
+	  `(,(r op-name) ,(char-set->scheme (apply op csets) r)
+	    . ,code-chunks))
+	 ((pair? (cdr code-chunks)) `(,(r op-name) . ,code-chunks))
+	 (else (car code-chunks)))	; Just one.
 	(apply op csets))))
 
 ;;; Parse a (/ <range-spec> ...) char-class into a character set in
@@ -360,54 +392,83 @@
 	(%re-bol (r 're-bol))	(%re-eol (r 're-eol))
 	(%list   (r 'list)))
 
-  (let recur ((re re))
-    ;; If (fetch-posix re) = #f, produce (OP . ARGS);
-    ;; Otherwise, produce (OP/POSIX ,@ARGS '<posix-translation>).
-    (define (doit op op/posix args fetch-posix)
-      (? ((fetch-posix re) =>
-	  (lambda (psx) `(,(r op/posix) ,@args
-					',(cre:string psx) ',(cre:tvec psx))))
-					
+    (let recur ((re re))
+      ;; If (fetch-posix re) = #f, produce (OP . ARGS);
+      ;; Otherwise, produce (OP/POSIX ,@ARGS '<posix-translation>).
+      (define (doit op op/tsm maybe-tsm op/posix args fetch-posix)
+	(cond
+	 ((fetch-posix re) =>
+	  (lambda (psx)
+	    `(,(r op/posix) ,@args ,maybe-tsm
+	      ',(cre:string psx) ',(cre:tvec psx))))
+       
+	 ((number? maybe-tsm)
+	  `(,(r op/tsm) ,@args ,maybe-tsm))
+	 (else
+	  `(,(r op) ,@args))))
+
+      (define (doit/leaf op op/posix args fetch-posix)
+	(cond
+	 ((fetch-posix re) =>
+	  (lambda (psx)
+	    `(,(r op/posix) ,@args
+	      ',(cre:string psx) ',(cre:tvec psx))))
+	 
 	 (else `(,(r op) . ,args))))
 
-    (? ((re-string? re)   (if (re-trivial? re) (r 're-trivial) ; Special hack
-			      (doit 'make-re-string 'make-re-string/posix
-				    `(,(re-string:chars re))
-				    re-string:posix)))
+      (cond
+       ((re-string? re)
+	(if (re-trivial? re)
+	    (r 're-trivial)		; Special hack
+	    (doit/leaf 'make-re-string 'make-re-string/posix
+		       `(,(re-string:chars re))
+		       re-string:posix)))
 
-       ((re-seq? re)      (doit '%make-re-seq '%make-re-seq/posix
-				`((,%list . ,(map recur (re-seq:elts re)))
-				  ,(re-seq:tsm re))
-				re-seq:posix))
+       ((re-seq? re)
+	(doit 'make-re-seq 'make-re-seq/tsm (re-seq:tsm re)
+	      'make-re-seq/posix
+	      `((,%list ,@(map recur (re-seq:elts re))))
+	      re-seq:posix))
 
-       ((re-choice? re)   (doit '%make-re-choice '%make-re-choice/posix
-				`((,%list . ,(map recur (re-choice:elts re)))
-				  ,(re-choice:tsm re))
-				re-choice:posix))
+       ((re-choice? re)
+	(doit 'make-re-choice 'make-re-choice/tsm (re-choice:tsm re)
+	      'make-re-choice/posix
+	      `((,%list ,@(map recur (re-choice:elts re))))
+	      re-choice:posix))
 
-       ((re-char-set? re) (if (re-any? re) (r 're-any) ; Special hack for ANY.
-			      (doit 'make-re-char-set 'make-re-char-set/posix
-				    `(,(char-set->scheme (re-char-set:cset re) r))
-				    re-char-set:posix)))
+       ((re-char-set? re)
+	(cond
+	 ((re-any? re)
+	  (r 're-any))			; Special hack for ANY.
+	 ((re-empty? re)
+	  (r 're-empty))		; Special hack for EMPTY.
+	 (else
+	  (doit/leaf 'make-re-char-set 'make-re-char-set/posix
+		     `(,(char-set->scheme (re-char-set:cset re) r))
+		     re-char-set:posix))))
 
-       ((re-repeat? re)   (doit '%make-re-repeat '%make-re-repeat/posix
-				`(,(re-repeat:from re)
-				  ,(re-repeat:to re)
-				  ,(recur (re-repeat:body re))
-				  ,(re-repeat:tsm re))
-				re-repeat:posix))
+       ((re-repeat? re)
+	(doit 'make-re-repeat 'make-re-repeat/tsm (re-repeat:tsm re)
+	      'make-re-repeat/posix
+	      `(,(re-repeat:from re)
+		,(re-repeat:to re)
+		,(recur (re-repeat:body re)))
+	      re-repeat:posix))
 
-       ((re-dsm? re)      (doit '%make-re-dsm '%make-re-dsm/posix
-				`(,(recur (re-dsm:body re))
-				  ,(re-dsm:pre-dsm  re)
-				  ,(re-dsm:tsm re))
-				re-dsm:posix))
+       ((re-dsm? re)
+	(doit 'make-re-dsm 'make-re-dsm/tsm (re-dsm:tsm re)
 
-       ((re-submatch? re) (doit '%make-re-submatch '%make-re-submatch/posix
-				`(,(recur (re-submatch:body re))
-				  ,(re-submatch:pre-dsm re)
-				  ,(re-submatch:tsm re))
-				re-submatch:posix))
+	      'make-re-dsm/posix
+	      `(,(recur (re-dsm:body re))
+		,(re-dsm:pre-dsm  re))
+	      re-dsm:posix))
+
+       ((re-submatch? re)
+	(doit 'make-re-submatch 'make-re-submatch/tsm (re-submatch:tsm re)
+	      'make-re-submatch/posix
+	      `(,(recur (re-submatch:body re))
+		,(re-submatch:pre-dsm re))
+	      re-submatch:posix))
 
        ((re-bos? re) %re-bos)
        ((re-eos? re) %re-eos)
@@ -438,17 +499,17 @@
     (if a
 	(if biga
 	    (if space 
-		(and one (switch char-set= cs
-			   ((char-set:full)		full)
-			   ((nonl-chars)		nonl)
-			   ((char-set:printing)		print)
-			   ((char-set:ascii)		ascii)
+		(and one (cond 
+			   ((char-set= cs char-set:full) full)
+			   ((char-set= cs nonl-chars) nonl)
+			   ((char-set= cs char-set:printing) print)
+			   ((char-set= cs char-set:ascii) ascii)
 			   (else #f)))
 		(if one
-		    (switch char-set= cs
-		      ((char-set:letter+digit)	alphanum)
-		      ((char-set:graphic)	graph)
-		      ((char-set:hex-digit)	hex)
+		    (cond
+		      ((char-set= cs char-set:letter+digit) alphanum)
+		      ((char-set= cs char-set:graphic) graph)
+		      ((char-set= cs char-set:hex-digit) hex)
 		      (else #f))
 		    (and (char-set= cs char-set:letter) alpha)))
 	    (and (char-set= cs char-set:lower-case) lower)) ; a, not A
@@ -458,13 +519,13 @@
 	    (if one
 		(and (not space) (char-set= cs char-set:digit) num)
 		(if space
-		    (switch char-set= cs
-		      ((char-set:whitespace) white)
-		      ((char-set:blank)      blank)
+		    (cond
+		      (( char-set= cs char-set:whitespace) white)
+		      (( char-set= cs char-set:blank) blank)
 		      (else #f))
-		    (switch char-set= cs
-		      ((char-set:punctuation)	punct)
-		      ((char-set:iso-control)	ctl)
+		    (cond
+		      ((char-set= cs char-set:punctuation) punct)
+		      ((char-set= cs char-set:iso-control) ctl)
 		      (else #f))))))))
 		
 
@@ -479,20 +540,21 @@
 				      'char-set:printing     'char-set:iso-control
 				      'char-set:hex-digit    'char-set:blank
 				      'char-set:ascii))))
-    (? ((not (char-set? cs)) cs) ; Dynamic -- *already* Scheme code.
-       ((char-set-empty? cs) (r 'char-set:empty))
-       ((try cs) => r)
-       ((try (char-set-complement cs)) =>
-	(lambda (name) `(,(r 'char-set-complement) ,name)))
+    (cond
+     ((not (char-set? cs)) cs)		; Dynamic -- *already* Scheme code.
+     ((char-set-empty? cs) (r 'char-set:empty))
+     ((try cs) => r)
+     ((try (char-set-complement cs)) =>
+      (lambda (name) `(,(r 'char-set-complement) ,name)))
 
-       (else
-	(receive (loose+ ranges+) (char-set->in-pair cs)
-	  (receive (loose- ranges-) (char-set->in-pair (char-set-complement cs))
-	    (let ((makeit (r 'spec->char-set)))
-	      (if (< (+ (length loose-) (* 12 (length ranges-)))
-		     (+ (length loose+) (* 12 (length ranges+))))
-		  `(,makeit #f ,(list->string loose-) ',ranges-)
-		  `(,makeit #t ,(list->string loose+) ',ranges+)))))))))
+     (else
+      (receive (loose+ ranges+) (char-set->in-pair cs)
+	(receive (loose- ranges-) (char-set->in-pair (char-set-complement cs))
+	  (let ((makeit (r 'spec->char-set)))
+	    (if (< (+ (length loose-) (* 12 (length ranges-)))
+		   (+ (length loose+) (* 12 (length ranges+))))
+		`(,makeit #f ,(list->string loose-) ',ranges-)
+		`(,makeit #t ,(list->string loose+) ',ranges+)))))))))
 
 
 
@@ -511,20 +573,21 @@
 					  'hex-digit    'blank
 					  'ascii)))
 	    (nchars (char-set-size cs)))
-	(? ((zero? nchars) `(,(r '|)))
-	   ((= 1 nchars) (apply string (char-set->list cs)))
-	   ((try cs) => r)
-	   ((try (char-set-complement cs)) =>
-	    (lambda (name) `(,(r '~) ,name)))
-	   (else (receive (cs rp comp?) (char-set->in-sexp-spec cs)
-		   (let ((args (append (? ((string=? cs "") '())
-					  ((= 1 (string-length cs)) `(,cs))
-					  (else `((,cs))))
-				       (if (string=? rp "") '()
-					   (list `(,(r '/) ,rp))))))
-		     (if (and (= 1 (length args)) (not comp?))
-			 (car args)
-			 `(,(r (if comp? '~ '|)) . ,args)))))))
+	(cond
+	 ((zero? nchars) `(,(r '|)))
+	 ((= 1 nchars) (apply string (char-set->list cs)))
+	 ((try cs) => r)
+	 ((try (char-set-complement cs)) =>
+	  (lambda (name) `(,(r '~) ,name)))
+	 (else (receive (cs rp comp?) (char-set->in-sexp-spec cs)
+		 (let ((args (append (cond ((string=? cs "") '())
+					   ((= 1 (string-length cs)) `(,cs))
+					   (else `((,cs))))
+				     (if (string=? rp "") '()
+					 (list `(,(r '/) ,rp))))))
+		   (if (and (= 1 (length args)) (not comp?))
+		       (car args)
+		       `(,(r (if comp? '~ '|)) . ,args)))))))
 
       `(,(r 'unquote) ,cs))) ; dynamic -- ,<cset-exp>
 
@@ -546,44 +609,45 @@
 
 (define (regexp->sre/renamer re r)
   (let recur ((re re))
-    (? ((re-string? re) (re-string:chars re))
+    (cond
+     ((re-string? re) (re-string:chars re))
 
-       ((re-seq? re)    `(,(r ':) . ,(regexp->sres/renamer re r)))
+     ((re-seq? re)    `(,(r ':) . ,(regexp->sres/renamer re r)))
 	 
-       ((re-choice? re)
-	(let ((elts (re-choice:elts re))
-	      (%| (r '|)))
-	  (if (pair? elts)
-	      `(,%| . ,(map recur elts))
-	      (let ((tsm  (re-choice:tsm  re)))
-		(if (zero? tsm) `(,%|) `(,(r 'dsm) ,tsm 0 (,%|)))))))
+     ((re-choice? re)
+      (let ((elts (re-choice:elts re))
+	    (%| (r '|)))
+	(if (pair? elts)
+	    `(,%| . ,(map recur elts))
+	    (let ((tsm  (re-choice:tsm  re)))
+	      (if (zero? tsm) `(,%|) `(,(r 'dsm) ,tsm 0 (,%|)))))))
 
-       ((re-char-set? re) (char-set->sre (re-char-set:cset re) r))
+     ((re-char-set? re) (char-set->sre (re-char-set:cset re) r))
 
-       ((re-repeat? re)
-	(let ((from (re-repeat:from re))
-	      (to (re-repeat:to re))
-	      (bodies (regexp->sres/renamer (re-repeat:body re) r)))
-	  (? ((and (eqv? from 0) (not to))    `(,(r '*) . ,bodies))
-	     ((and (eqv? from 0) (eqv? to 1)) `(,(r '?) . ,bodies))
-	     ((and (eqv? from 1) (not to))    `(,(r '+) . ,bodies))
-	     ((eqv? from to)		      `(,(r '=) ,to . bodies))
-	     (to                           `(,(r '**) ,from ,to . ,bodies))
-	     (else                         `(,(r '>=) ,from . ,bodies)))))
+     ((re-repeat? re)
+      (let ((from (re-repeat:from re))
+	    (to (re-repeat:to re))
+	    (bodies (regexp->sres/renamer (re-repeat:body re) r)))
+	(cond ((and (eqv? from 0) (not to))    `(,(r '*) . ,bodies))
+	      ((and (eqv? from 0) (eqv? to 1)) `(,(r '?) . ,bodies))
+	      ((and (eqv? from 1) (not to))    `(,(r '+) . ,bodies))
+	      ((eqv? from to)		      `(,(r '=) ,to . bodies))
+	      (to                           `(,(r '**) ,from ,to . ,bodies))
+	      (else                         `(,(r '>=) ,from . ,bodies)))))
 
-       ((re-dsm? re)
-	`(,(r 'dsm) ,(re-dsm:pre-dsm re) ,(re-dsm:post-dsm re)
-		    . ,(regexp->sres/renamer (re-dsm:body re) r)))
+     ((re-dsm? re)
+      `(,(r 'dsm) ,(re-dsm:pre-dsm re) ,(re-dsm:post-dsm re)
+	. ,(regexp->sres/renamer (re-dsm:body re) r)))
 
-       ((re-submatch? re)
-	`(,(r 'submatch) . ,(regexp->sres/renamer (re-submatch:body re) r)))
+     ((re-submatch? re)
+      `(,(r 'submatch) . ,(regexp->sres/renamer (re-submatch:body re) r)))
 
-       ((re-bos? re) (r 'bos))
-       ((re-eos? re) (r 'eos))
-       ((re-bol? re) (r 'bol))
-       ((re-eol? re) (r 'eol))
+     ((re-bos? re) (r 'bos))
+     ((re-eos? re) (r 'eos))
+     ((re-bol? re) (r 'bol))
+     ((re-eol? re) (r 'eol))
 
-       (else re))))			; Presumably it's code.
+     (else re))))			; Presumably it's code.
 
 (define (regexp->sre re) (regexp->sre/renamer re (lambda (x) x)))
 
