@@ -44,27 +44,12 @@
 
 (define (dump-scsh-program start filename)
   (let ((context (user-context)))
-  (really-dump-scsh-program (lambda (args)
-			      (with-scsh-sighandlers 
-			       #f
-			       (lambda ()
-				 (with-autoreaping
-				  (lambda ()
-				    (install-env)
-				    (initialize-cwd)
-				    (init-scsh-vars #f)	; Do it quietly.
-				    (start-new-session context    
-						       (current-input-port)
-						       (current-output-port)
-						       (current-error-port)
-						       args
-						       #f)
-				    (with-interaction-environment     
-				     (user-environment)
-				     (lambda ()
-				       (start args))))))))
-				  filename)))
-
+    (really-dump-scsh-program (lambda (args)
+				(with-scsh-initialized
+				 #f context args 
+				 (lambda ()
+				   (start args))))
+			      filename)))
 
 (define (scsh-stand-alone-resumer start)
   (usual-resumer			; sets up exceptions, interrupts,
@@ -80,7 +65,7 @@
 	 (lambda ()
 	   (let ((dynamic-env (get-dynamic-env))
 		 (*result* 4711))
-	     (let ((runnable (make-thread-queue))
+	     (let ((runnable (make-queue))
 		   (thread (make-thread (lambda ()
 					  (set! *result*
 						(start (command-line))))
@@ -88,7 +73,7 @@
 					'scsh-initial-thread))
 		   (thread-count (make-counter)))
 	      
-	       (enqueue-thread! runnable thread)
+	       (enqueue! runnable thread)
 	       (increment-counter! thread-count)
 	      
 	       (run-threads
