@@ -14,6 +14,7 @@
 #include <fcntl.h>		/* for O_RDWR */
 #include <sys/stat.h>
 #include <sys/param.h> /* For gethostname() */
+#include <sys/utsname.h> /* for uname() */
 
 #include <netdb.h>
 /* This lossage brought to you by Solaris and BIND */
@@ -36,6 +37,7 @@
 #include <crypt.h> 
 #endif
 #include "cstuff.h"
+#include <limits.h>     /* NGROUPS_MAX on solaris */
 
 /* Make sure our exports match up w/the implementation: */
 #include "syscalls1.h"
@@ -59,17 +61,21 @@ extern char **environ;
 s48_value wait_pid(s48_value s48_pid, s48_value s48_flags)
 {
   int status=0;
-  pid_t pid = (pid_t) s48_extract_integer (s48_pid); 
-  int flags = s48_extract_integer (s48_flags); 
+  pid_t pid;
+  int flags;
   pid_t result_pid;
   s48_value sch_retval = S48_UNSPECIFIC;
 
   s48_value sch_result_pid = S48_UNSPECIFIC;
   s48_value sch_status = S48_UNSPECIFIC;
   s48_value sch_status_list = S48_UNSPECIFIC;
-  S48_DECLARE_GC_PROTECT(3);
+  S48_DECLARE_GC_PROTECT(5);
 
-  S48_GC_PROTECT_3(sch_result_pid, sch_status, sch_status_list);
+  S48_GC_PROTECT_5(s48_pid, s48_flags,
+		   sch_result_pid, sch_status, sch_status_list);
+
+  pid = (pid_t) s48_extract_integer (s48_pid); 
+  flags = s48_extract_integer (s48_flags); 
 
   result_pid = waitpid(pid, &status, flags);
   if (result_pid == -1)
@@ -134,8 +140,8 @@ s48_value scheme_exec(s48_value prog, s48_value argv, s48_value env)
     Free(unix_env);
   }
   Free(unix_argv);
-  return s48_enter_fixnum(e); /* Don't raise an exception since
-				 we're maybe just testing the path */
+  return s48_enter_integer(e); /* Don't raise an exception since
+				  we're maybe just testing the path */
 }
 
 s48_value scsh_exit(s48_value status)
@@ -189,7 +195,7 @@ s48_value scsh_kill (s48_value sch_pid, s48_value sch_signal)
   
 
 /* Read the symlink. */
-
+#ifdef MAXPATHLEN
 s48_value scsh_readlink(s48_value sch_path)
 {
   char linkpath[MAXPATHLEN+1];
@@ -202,6 +208,37 @@ s48_value scsh_readlink(s48_value sch_path)
       return s48_enter_string(linkpath);
     }
 }
+#else
+s48_value scsh_readlink(s48_value sch_path)
+{
+  char *linkpath;
+  int size;
+  int retval;
+  s48_value sch_sym_link_path = S48_UNSPECIFIC;
+  
+  for (size = 256;; size *=2){
+
+    linkpath = Malloc(char,size);
+    if (!linkpath)
+      s48_raise_os_error_1(errno, sch_path);
+
+    retval = readlink(s48_extract_string (sch_path), linkpath, size);
+
+    if (retval == -1){
+      free (linkpath);
+      s48_raise_os_error_1(errno, sch_path);
+    }
+    
+    if (retval < size){
+      sch_sym_link_path = s48_enter_substring (linkpath, retval);
+      free (linkpath);
+      return sch_sym_link_path;
+    }
+    free (linkpath);
+  }
+}
+#endif
+
 
 s48_value scsh_rename(s48_value sch_from, s48_value sch_to)
 {
@@ -228,12 +265,19 @@ s48_value scsh_rmdir(s48_value sch_path)
 
 s48_value scm_utime(s48_value sch_path, s48_value sch_ac, s48_value sch_mod)
 {
-    struct utimbuf t;
-    int retval;
-    t.actime = s48_extract_integer (sch_ac);
-    t.modtime = s48_extract_integer (sch_mod);
-    retval = utime(s48_extract_string (sch_path), &t);
-    if (retval == -1)
+  struct utimbuf t;
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_path, sch_ac, sch_mod);
+
+  t.actime = s48_extract_integer (sch_ac);
+  t.modtime = s48_extract_integer (sch_mod);
+  retval = utime(s48_extract_string (sch_path), &t);
+
+  S48_GC_UNPROTECT();
+
+  if (retval == -1)
     s48_raise_os_error_3(errno, sch_path, sch_ac, sch_mod);
   return S48_UNSPECIFIC;
 }
@@ -291,6 +335,7 @@ s48_value scheme_cwd()
 {
   char *buf;
   int size = scsh_path_max + 1; /* +1 for terminating nul byte... */
+  s48_value cwd = S48_UNSPECIFIC;
 
   buf = Malloc(char,size);
   if(!buf) goto lose;
@@ -303,8 +348,11 @@ s48_value scheme_cwd()
       if( !nbuf ) goto lose;
       buf = nbuf;
     }
-
-  return s48_enter_string(buf);		/* win */
+  cwd = s48_enter_string(buf);		/* win */
+  
+  Free(buf);
+  
+  return cwd;
 
  lose:
   {int e = errno;
@@ -378,8 +426,16 @@ s48_value cpu_clock_ticks_per_sec()
 
 s48_value scsh_chmod(s48_value sch_path, s48_value sch_mode)
 {
-  int retval = chmod (s48_extract_string(sch_path),
-		      s48_extract_integer(sch_mode));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_path, sch_mode);
+
+  retval = chmod (s48_extract_string(sch_path),
+		  s48_extract_integer(sch_mode));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_path, sch_mode);
   return S48_UNSPECIFIC;
@@ -387,8 +443,16 @@ s48_value scsh_chmod(s48_value sch_path, s48_value sch_mode)
 
 s48_value scsh_fchmod(s48_value sch_fd, s48_value sch_mode)
 {
-  int retval = fchmod (s48_extract_fixnum(sch_fd),
-		       s48_extract_integer(sch_mode));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_fd, sch_mode);
+
+  retval = fchmod (s48_extract_fixnum(sch_fd),
+		   s48_extract_integer(sch_mode));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_fd, sch_mode);
   return S48_UNSPECIFIC;
@@ -396,10 +460,17 @@ s48_value scsh_fchmod(s48_value sch_fd, s48_value sch_mode)
 
 s48_value scsh_chown(s48_value sch_path, s48_value sch_uid, s48_value sch_gid)
 {
-  int retval = chown(s48_extract_string(sch_path),
-		     s48_extract_integer(sch_uid),
-		     s48_extract_integer(sch_gid));
-  
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_path, sch_uid, sch_gid);
+
+  retval = chown(s48_extract_string(sch_path),
+		 s48_extract_integer(sch_uid),
+		 s48_extract_integer(sch_gid));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_3(errno, sch_path, sch_uid, sch_gid);
   return S48_UNSPECIFIC;
@@ -407,9 +478,16 @@ s48_value scsh_chown(s48_value sch_path, s48_value sch_uid, s48_value sch_gid)
 
 s48_value scsh_fchown(s48_value sch_fd, s48_value sch_uid, s48_value sch_gid)
 {
-  int retval = fchown(s48_extract_fixnum(sch_fd),
-		      s48_extract_integer(sch_uid),
-		      s48_extract_integer(sch_gid));
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_fd, sch_uid, sch_gid);
+
+  retval = fchown(s48_extract_fixnum(sch_fd),
+		  s48_extract_integer(sch_uid),
+		  s48_extract_integer(sch_gid));
+
+  S48_GC_UNPROTECT();
   
   if (retval == -1)
     s48_raise_os_error_3(errno, sch_fd, sch_uid, sch_gid);
@@ -418,8 +496,16 @@ s48_value scsh_fchown(s48_value sch_fd, s48_value sch_uid, s48_value sch_gid)
 
 s48_value scsh_access(s48_value sch_path, s48_value sch_mode)
 {
-  int retval = access (s48_extract_string(sch_path),
-		       s48_extract_integer(sch_mode));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_path, sch_mode);
+
+  retval = access (s48_extract_string(sch_path),
+		   s48_extract_integer(sch_mode));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_path, sch_mode);
   return S48_UNSPECIFIC;
@@ -534,8 +620,16 @@ s48_value scsh_symlink(s48_value sch_name1, s48_value sch_name2)
 
 s48_value scsh_truncate(s48_value sch_path, s48_value sch_length)
 {
-  int retval = truncate (s48_extract_string (sch_path),
-			 s48_extract_integer (sch_length));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_path, sch_length);
+
+  retval = truncate (s48_extract_string (sch_path),
+		     s48_extract_integer (sch_length));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_path, sch_length);
   return S48_UNSPECIFIC;
@@ -543,8 +637,17 @@ s48_value scsh_truncate(s48_value sch_path, s48_value sch_length)
 
 s48_value scsh_ftruncate(s48_value sch_fdes, s48_value sch_length)
 {
-  int retval = ftruncate (s48_extract_fixnum (sch_fdes),
-			  s48_extract_integer (sch_length));
+  int retval;
+
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_fdes, sch_length);
+
+  retval = ftruncate (s48_extract_fixnum (sch_fdes),
+		      s48_extract_integer (sch_length));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_fdes, sch_length);
   return S48_UNSPECIFIC;
@@ -602,9 +705,17 @@ s48_value scsh_dup2(s48_value sch_oldd,  s48_value sch_newd)
 s48_value scsh_lseek(s48_value sch_fdes, s48_value sch_offset, 
 		     s48_value sch_whence)
 {
-  int retval = lseek (s48_extract_fixnum (sch_fdes),
-		      s48_extract_integer (sch_offset),
-		      s48_extract_fixnum (sch_whence));
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_fdes, sch_offset, sch_whence);
+
+  retval = lseek (s48_extract_fixnum (sch_fdes),
+		  s48_extract_integer (sch_offset),
+		  s48_extract_fixnum (sch_whence));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_3 (errno, sch_fdes, sch_offset, sch_whence);
   return s48_enter_integer (retval);
@@ -659,7 +770,15 @@ s48_value scsh_getegid()
 
 s48_value scsh_setgid(s48_value gid)
 {
-  int retval = setgid (s48_extract_integer (gid));
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(gid);
+
+  retval = setgid (s48_extract_integer (gid));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, gid);
   return S48_UNSPECIFIC;
@@ -667,11 +786,19 @@ s48_value scsh_setgid(s48_value gid)
 
 s48_value scsh_setegid(s48_value gid)
 {
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(gid);
+
 #ifdef HAVE_SETEGID
-  int retval = setegid (s48_extract_integer (gid));
+  retval = setegid (s48_extract_integer (gid));
 #else 
-  int retval = setregid (-1, s48_extract_integer (gid));
+  retval = setregid (-1, s48_extract_integer (gid));
 #endif
+  
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, gid);
   return S48_UNSPECIFIC;
@@ -682,33 +809,20 @@ s48_value scsh_setegid(s48_value gid)
 s48_value get_groups()
 {
   int retval;
-  int veclen = getgroups(0,NULL);
+  gid_t gvec[NGROUPS_MAX + 1];
+  int veclen = getgroups(NGROUPS_MAX + 1, gvec);
   int i = veclen;
   s48_value l = S48_NULL;
-  gid_t gvec0[20], *gp = gvec0;
 
-  if( veclen > 20 )
-    if (NULL == (gp=Malloc(gid_t,veclen))) s48_raise_os_error(errno);
-
-  retval = getgroups(veclen, gp);
-
-  if (retval == -1){
-    if (veclen > 20) Free(gp);
+  if (veclen == -1)
     s48_raise_os_error(errno);
-  }
-  else if (retval != veclen){     /* paranoia... */
-    get_groups();
-  }
-  else{
+  else {
     S48_DECLARE_GC_PROTECT(1);
     S48_GC_PROTECT_1(l);
 
-    while (i > 0){
-      l = s48_cons(s48_enter_integer(gp[--i]), l);
-    }
+    while (i > 0)
+      l = s48_cons(s48_enter_integer(gvec[--i]), l);
     
-    if (veclen > 20) Free(gp);
-
     S48_GC_UNPROTECT();
 
     return l;
@@ -726,7 +840,15 @@ s48_value scsh_geteuid()
 
 s48_value scsh_setuid(s48_value uid)
 {
-  int retval = setuid (s48_extract_integer (uid));
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(uid);
+
+  retval = setuid (s48_extract_integer (uid));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, uid);
   return S48_UNSPECIFIC;
@@ -734,11 +856,19 @@ s48_value scsh_setuid(s48_value uid)
 
 s48_value scsh_seteuid(s48_value uid)
 {
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(uid);
+
 #ifdef HAVE_SETEUID
-  int retval = seteuid (s48_extract_integer (uid));
+  retval = seteuid (s48_extract_integer (uid));
 #else
-  int retval = setreuid (-1, s48_extract_integer (uid));
+  retval = setreuid (-1, s48_extract_integer (uid));
 #endif
+  
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, uid);
   return S48_UNSPECIFIC;
@@ -761,8 +891,16 @@ s48_value scsh_getpgrp()
 
 s48_value scsh_setpgid(s48_value sch_pid, s48_value sch_pgrp)
 {
-  int retval = setpgid(s48_extract_integer(sch_pid),
-		       s48_extract_integer(sch_pgrp));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_pid, sch_pgrp);
+
+  retval = setpgid(s48_extract_integer(sch_pid),
+		   s48_extract_integer(sch_pgrp));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_pid, sch_pgrp);
   return S48_UNSPECIFIC;
@@ -931,8 +1069,16 @@ s48_value errno_msg(s48_value sch_i)
 
 s48_value fcntl_read(s48_value fd, s48_value command)
 { 
-  int ret = fcntl(s48_extract_fixnum (fd), 
-				 s48_extract_integer (command)); 
+  int ret;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(fd, command);
+
+  ret = fcntl(s48_extract_fixnum (fd), 
+	      s48_extract_integer (command)); 
+
+  S48_GC_UNPROTECT();
+
   if (ret == -1)
     s48_raise_os_error_2(errno, fd, command);
   else return s48_enter_fixnum (ret); 
@@ -941,9 +1087,18 @@ s48_value fcntl_read(s48_value fd, s48_value command)
 
 s48_value fcntl_write(s48_value fd, s48_value command, s48_value value)
 { 
-  int ret = fcntl(s48_extract_fixnum (fd), 
-		  s48_extract_integer (command), 
-		  s48_extract_integer (value));
+  int ret;
+
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(fd, command, value);
+
+  ret = fcntl(s48_extract_fixnum (fd), 
+	      s48_extract_integer (command), 
+	      s48_extract_integer (value));
+
+  S48_GC_UNPROTECT();
+  
   if (ret == -1)
     s48_raise_os_error_3(errno, fd, command, value);
   else return s48_enter_fixnum (ret); 
@@ -963,6 +1118,28 @@ s48_value scm_crypt(s48_value key, s48_value salt)
   return s48_enter_string (ret);
 }
 
+static s48_value uname_record_type_binding = S48_FALSE;
+
+s48_value scm_uname(void)
+{
+  s48_value uname_record = S48_UNSPECIFIC;
+  struct utsname uname_struct;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(uname_record);
+  if (uname(&uname_struct) == -1)
+    s48_raise_os_error(errno);
+  
+  uname_record = s48_make_record(uname_record_type_binding);
+  S48_RECORD_SET(uname_record, 0, s48_enter_string (uname_struct.sysname));
+  S48_RECORD_SET(uname_record, 1, s48_enter_string (uname_struct.nodename));
+  S48_RECORD_SET(uname_record, 2, s48_enter_string (uname_struct.release));
+  S48_RECORD_SET(uname_record, 3, s48_enter_string (uname_struct.version));
+  S48_RECORD_SET(uname_record, 4, s48_enter_string (uname_struct.machine));
+  S48_GC_UNPROTECT();
+  return uname_record;
+}
+
 void s48_init_syscalls (){
   S48_EXPORT_FUNCTION(scheme_exec);
   S48_EXPORT_FUNCTION(scsh_exit);
@@ -1028,10 +1205,13 @@ void s48_init_syscalls (){
   S48_EXPORT_FUNCTION(scm_gethostname);
   S48_EXPORT_FUNCTION(errno_msg);
   S48_EXPORT_FUNCTION(scm_crypt);
+  S48_EXPORT_FUNCTION(scm_uname);
   S48_GC_PROTECT_GLOBAL(envvec_record_type_binding);
   S48_GC_PROTECT_GLOBAL(add_envvec_finalizerB_binding);
   S48_GC_PROTECT_GLOBAL(current_env);
+  S48_GC_PROTECT_GLOBAL(uname_record_type_binding);
   envvec_record_type_binding = s48_get_imported_binding("envvec-record-type");
+  uname_record_type_binding = s48_get_imported_binding("uname-record-type");
   add_envvec_finalizerB_binding = 
     s48_get_imported_binding("add-envvec-finalizer!");
 }
