@@ -3,7 +3,7 @@
 */
 
 /* The source code is conditionalised by three #ifdef feature macros:
-** HAVE_TZNAME
+** HAVE_STRUCT_TM_TM_ZONE/HAVE_TZNAME
 **   The char *tzname[2] global variable is POSIX. Everyone provides
 **   it...except some "classic" versions of SunOS that we still care about
 **   running (People in LCS/AI refuse to switch to Solaris). So, we kluge
@@ -135,7 +135,7 @@ s48_value time2date(s48_value sch_t, s48_value sch_zone)
 	int offset = s48_extract_fixnum(sch_zone);
 	t += s48_extract_fixnum(sch_zone);
 	d = *gmtime(&t);
-	sch_tz_name = s48_enter_string("");
+	sch_tz_name = S48_FALSE;
 	sch_tz_secs = s48_enter_fixnum (offset);
 	}
     else {
@@ -161,10 +161,12 @@ s48_value time2date(s48_value sch_t, s48_value sch_zone)
 	** that happens.
 	*/
 	{ int error = 0;
-#ifndef HAVE_TZNAME
-	  char *zone = d.tm_zone; /* Hack it for SunOS. */
-#else
+#ifdef HAVE_TZNAME
 	  char *zone = tzname[d.tm_isdst];
+#else
+#ifdef HAVE_STRUCT_TM_TM_ZONE
+	  const char *zone = d.tm_zone; /* Hack it for SunOS. */
+#endif
 #endif
 	  char *newzone = Malloc(char, 1+strlen(zone));
 	  if( newzone ){
@@ -250,7 +252,7 @@ s48_value date2time(s48_value sec, s48_value min, s48_value hour,
     else if( S48_STRING_P(tz_name) ) {	/* Time zone */
 	char *newenv[2];
 	char **oldenv = make_newenv(tz_name, newenv);
-	if( !oldenv ) return s48_enter_fixnum(errno);
+	if( !oldenv ) return s48_enter_integer(errno);
 	tzset(); /* NetBSD, SunOS POSIX-noncompliance requires this. */
 	errno = 0;
 
@@ -330,7 +332,7 @@ s48_value format_date(s48_value sch_fmt, s48_value sch_sec, s48_value sch_min,
     d.tm_yday = s48_extract_fixnum(sch_year_day);
     d.tm_isdst = (S48_EQ_P (sch_summer, S48_FALSE)) ? 0 : 1;
 
-#ifdef HAVE_TM_ZONE
+#ifdef HAVE_STRUCT_TM_TM_ZONE
     d.tm_zone = s48_extract_string(sch_tz); /* FreeBSD's strftime reads this */
 #endif
 
@@ -392,7 +394,6 @@ s48_value format_date(s48_value sch_fmt, s48_value sch_sec, s48_value sch_min,
     if( zone && S48_STRING_P(sch_tz) ) {
 	oldenv = make_newenv(sch_tz, newenv);
 	if( !oldenv ) {
-	    int err = errno;
 	    Free(fmt);
 	    s48_raise_os_error_1(errno, sch_fmt);
 	    }
@@ -412,6 +413,7 @@ s48_value format_date(s48_value sch_fmt, s48_value sch_sec, s48_value sch_min,
 #endif
     sch_ans = s48_enter_string(target);
     Free(fmt2);
+    Free(target);
     if( oldenv ) revert_env(oldenv);
     return sch_ans;
 
